/// <reference types="node" />
import { Disk, ReadResult, WriteResult } from 'file-disk';
import BlockMap = require('blockmap');
import { Metadata } from '../metadata';
import { SourceDestination } from '../source-destination';
import { SourceSource } from '../source-source';
export declare type ConfigureFunction = (disk: Disk, config: any) => Promise<void>;
export declare class SourceDisk extends Disk {
    private source;
    constructor(source: SourceDestination);
    _getCapacity(): Promise<number>;
    _read(buffer: Buffer, bufferOffset: number, length: number, fileOffset: number): Promise<ReadResult>;
    _write(buffer: Buffer, bufferOffset: number, length: number, fileOffset: number): Promise<WriteResult>;
    _flush(): Promise<void>;
}
export declare class ConfiguredSource extends SourceSource {
    private shouldTrimPartitions;
    private createStreamFromDisk;
    private config?;
    private disk;
    private configure?;
    constructor(source: SourceDestination, shouldTrimPartitions: boolean, createStreamFromDisk: boolean, configure?: ConfigureFunction | 'legacy', config?: any);
    private getBlockmap;
    canRead(): Promise<boolean>;
    canCreateReadStream(): Promise<boolean>;
    canCreateSparseReadStream(): Promise<boolean>;
    read(buffer: Buffer, bufferOffset: number, length: number, sourceOffset: number): Promise<ReadResult>;
    createReadStream(...args: any[]): Promise<NodeJS.ReadableStream>;
    private createSparseReadStreamFromDisk;
    private createSparseReadStreamFromStream;
    createSparseReadStream(generateChecksums: boolean): Promise<BlockMap.FilterStream | BlockMap.ReadStream>;
    _getMetadata(): Promise<Metadata>;
    private trimPartitions;
    protected _open(): Promise<void>;
    protected _close(): Promise<void>;
}
