"use strict";
/*
 * Copyright 2018 balena.io
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const bluebird_1 = require("bluebird");
const _ = require("lodash");
const udif_1 = require("udif");
const source_destination_1 = require("./source-destination");
const source_source_1 = require("./source-source");
const errors_1 = require("../errors");
const stream_limiter_1 = require("../stream-limiter");
class DmgSource extends source_source_1.SourceSource {
    constructor(source) {
        super(source);
        this.getMappedBlocksSize = () => {
            return _(this.image.resourceFork.blkx)
                .map('map.blocks')
                .flatten()
                .filter((blk) => [udif_1.BLOCK.RAW, udif_1.BLOCK.UDCO, udif_1.BLOCK.UDZO, udif_1.BLOCK.UDBZ, udif_1.BLOCK.LZFSE].indexOf(blk.type) !== -1)
                .map(blk => blk.sectorCount * udif_1.SECTOR_SIZE)
                .sum();
        };
        this.image = new udif_1.Image('', { fs: new source_destination_1.SourceDestinationFs(source) });
    }
    canCreateReadStream() {
        return __awaiter(this, void 0, void 0, function* () {
            return true;
        });
    }
    canCreateSparseReadStream() {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO: We act like we can't create sparse streams because we have no way to verify
            // flashed images.
            // This is because node-udif does not use blockmap but implements its own BlockMap
            // We need a function to extract a regular BlockMap from the UDIFImage.
            //return true;
            return false;
        });
    }
    createReadStream(emitProgress = false, start = 0, end) {
        return __awaiter(this, void 0, void 0, function* () {
            if (start !== 0) {
                throw new errors_1.NotCapable();
            }
            const stream = yield this.image.createReadStream();
            if (end !== undefined) {
                const transform = new stream_limiter_1.StreamLimiter(stream, end + 1);
                return transform;
            }
            return stream;
        });
    }
    createSparseReadStream(generateChecksums) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.image.createSparseReadStream();
        });
    }
    _getMetadata() {
        return __awaiter(this, void 0, void 0, function* () {
            const compressedSize = (yield this.source.getMetadata()).size;
            return {
                compressedSize,
                size: this.image.getUncompressedSize(),
                blockmappedSize: this.getMappedBlocksSize(),
            };
        });
    }
    _open() {
        const _super = Object.create(null, {
            _open: { get: () => super._open }
        });
        return __awaiter(this, void 0, void 0, function* () {
            yield _super._open.call(this);
            yield bluebird_1.promisify(this.image.open).bind(this.image)();
        });
    }
    _close() {
        const _super = Object.create(null, {
            _close: { get: () => super._close }
        });
        return __awaiter(this, void 0, void 0, function* () {
            yield bluebird_1.promisify(this.image.close).bind(this.image)();
            yield _super._close.call(this);
        });
    }
}
DmgSource.requiresRandomReadableSource = true;
DmgSource.mimetype = 'application/x-apple-diskimage';
exports.DmgSource = DmgSource;
source_destination_1.SourceDestination.register(DmgSource);
//# sourceMappingURL=dmg.js.map