/// <reference types="node" />
import { EventEmitter } from 'events';
import { ReadResult, WriteResult } from 'file-disk';
import BlockMap = require('blockmap');
import { SourceDestination, Verifier } from './source-destination';
import { SparseWriteStream } from '../sparse-write-stream';
export declare class MultiDestinationError extends Error {
    error: Error;
    destination: SourceDestination;
    constructor(error: Error, destination: SourceDestination);
}
export declare class MultiDestinationVerifier extends Verifier {
    private source;
    private verifiers;
    private timer;
    constructor(source: MultiDestination, checksumOrBlockmap: string | BlockMap, size?: number);
    private oneVerifierFinished;
    private emitProgress;
    run(): Promise<void>;
}
export declare class MultiDestination extends SourceDestination {
    readonly destinations: Set<SourceDestination>;
    readonly erroredDestinations: Set<SourceDestination>;
    constructor(destinations: SourceDestination[]);
    destinationError(destination: SourceDestination, error: Error, stream?: EventEmitter): void;
    readonly activeDestinations: Set<SourceDestination>;
    private can;
    canRead(): Promise<boolean>;
    canWrite(): Promise<boolean>;
    canCreateReadStream(): Promise<boolean>;
    canCreateSparseReadStream(): Promise<boolean>;
    canCreateWriteStream(): Promise<boolean>;
    canCreateSparseWriteStream(): Promise<boolean>;
    read(buffer: Buffer, bufferOffset: number, length: number, sourceOffset: number): Promise<ReadResult>;
    write(buffer: Buffer, bufferOffset: number, length: number, fileOffset: number): Promise<WriteResult>;
    createReadStream(...args: any[]): Promise<NodeJS.ReadableStream>;
    createSparseReadStream(...args: any[]): Promise<BlockMap.FilterStream | BlockMap.ReadStream>;
    private createStream;
    createWriteStream(): Promise<NodeJS.WritableStream>;
    createSparseWriteStream(): Promise<SparseWriteStream>;
    createVerifier(checksumOrBlockmap: string | BlockMap, size?: number): Verifier;
    protected _open(): Promise<void>;
    protected _close(): Promise<void>;
}
