/// <reference types="node" />
import { EventEmitter } from 'events';
import { ReadResult, WriteResult } from 'file-disk';
import { GetPartitionsResult } from 'partitioninfo';
import { Stream as HashStream } from 'xxhash';
import BlockMap = require('blockmap');
import { SourceSource } from './source-source';
import { SparseWriteStream } from '../sparse-write-stream';
import { Metadata } from './metadata';
import { ProgressEvent } from './progress';
export declare class CountingHashStream extends HashStream {
    bytesWritten: number;
    _transform(chunk: Buffer, encoding: string, callback: () => void): void;
}
export declare const ProgressHashStream: {
    new (...args: any[]): {
        _attributeValue: number;
        _attributeDelta: number;
        addListener(event: string | symbol, listener: Function): any;
        on(event: string | symbol, listener: Function): any;
        once(event: string | symbol, listener: Function): any;
        prependListener(event: string | symbol, listener: Function): any;
        prependOnceListener(event: string | symbol, listener: Function): any;
        removeListener(event: string | symbol, listener: Function): any;
        removeAllListeners(event?: string | symbol | undefined): any;
        setMaxListeners(n: number): any;
        getMaxListeners(): number;
        listeners(event: string | symbol): Function[];
        emit(event: string | symbol, ...args: any[]): boolean;
        eventNames(): (string | symbol)[];
        listenerCount(type: string | symbol): number;
    };
} & typeof CountingHashStream;
export declare function createHasher(): {
    _attributeValue: number;
    _attributeDelta: number;
    addListener(event: string | symbol, listener: Function): any;
    on(event: string | symbol, listener: Function): any;
    once(event: string | symbol, listener: Function): any;
    prependListener(event: string | symbol, listener: Function): any;
    prependOnceListener(event: string | symbol, listener: Function): any;
    removeListener(event: string | symbol, listener: Function): any;
    removeAllListeners(event?: string | symbol | undefined): any;
    setMaxListeners(n: number): any;
    getMaxListeners(): number;
    listeners(event: string | symbol): Function[];
    emit(event: string | symbol, ...args: any[]): boolean;
    eventNames(): (string | symbol)[];
    listenerCount(type: string | symbol): number;
} & CountingHashStream;
export declare class SourceDestinationFs {
    private source;
    constructor(source: SourceDestination);
    open(path: string, options: any, callback: (error: Error | null, fd?: number) => void): void;
    close(fd: number, callback: (error: Error | null) => void): void;
    fstat(fd: number, callback: (error: Error | null, stats?: {
        size: number;
    }) => void): void;
    read(fd: number, buffer: Buffer, bufferOffset: number, length: number, sourceOffset: number, callback: (error: Error | null, bytesRead?: number, buffer?: Buffer) => void): void;
}
export declare abstract class Verifier extends EventEmitter {
    progress: ProgressEvent;
    abstract run(): Promise<void>;
    protected handleEventsAndPipe(stream: NodeJS.ReadableStream, meter: NodeJS.WritableStream): void;
}
export declare class StreamVerifier extends Verifier {
    private source;
    private checksum;
    private size;
    constructor(source: SourceDestination, checksum: string, size: number);
    run(): Promise<void>;
}
export declare class SparseStreamVerifier extends Verifier {
    private source;
    private blockMap;
    constructor(source: SourceDestination, blockMap: BlockMap);
    private wrapErrorAndEmit;
    run(): Promise<void>;
}
export declare class SourceDestination extends EventEmitter {
    static readonly imageExtensions: string[];
    static readonly mimetype?: string;
    private static mimetypes;
    private metadata;
    private isOpen;
    static register(Cls: typeof SourceSource): void;
    canRead(): Promise<boolean>;
    canWrite(): Promise<boolean>;
    canCreateReadStream(): Promise<boolean>;
    canCreateSparseReadStream(): Promise<boolean>;
    canCreateWriteStream(): Promise<boolean>;
    canCreateSparseWriteStream(): Promise<boolean>;
    getMetadata(): Promise<Metadata>;
    _getMetadata(): Promise<Metadata>;
    read(buffer: Buffer, bufferOffset: number, length: number, sourceOffset: number): Promise<ReadResult>;
    write(buffer: Buffer, bufferOffset: number, length: number, fileOffset: number): Promise<WriteResult>;
    createReadStream(emitProgress?: boolean, start?: number, end?: number): Promise<NodeJS.ReadableStream>;
    createSparseReadStream(generateChecksums?: boolean): Promise<BlockMap.FilterStream | BlockMap.ReadStream>;
    createWriteStream(): Promise<NodeJS.WritableStream>;
    createSparseWriteStream(): Promise<SparseWriteStream>;
    open(): Promise<void>;
    close(): Promise<void>;
    protected _open(): Promise<void>;
    protected _close(): Promise<void>;
    createVerifier(checksumOrBlockmap: string | BlockMap, size?: number): Verifier;
    private getMimeTypeFromName;
    private getMimeTypeFromContent;
    private getMimetype;
    getInnerSource(): Promise<SourceDestination>;
    getPartitionTable(): Promise<GetPartitionsResult | undefined>;
}
